/*******************************************************************************
 * Copyright (c) 2011 Ericsson
 * 
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Bernd Hufmann - Initial API and implementation
 *******************************************************************************/
package org.eclipse.linuxtools.ust.sample.ui.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.linuxtools.lttng.event.LttngEvent;
import org.eclipse.linuxtools.tmf.event.TmfEvent;
import org.eclipse.linuxtools.tmf.ui.views.statistics.ITmfExtraEventInfo;
import org.eclipse.linuxtools.tmf.ui.views.statistics.model.TmfBaseStatisticsTree;
import org.eclipse.linuxtools.tmf.util.TmfFixedArray;

public class CustomStatisticsModel extends TmfBaseStatisticsTree {

    /**
     * <h4>Header for the event types categories.</h4>
     */
    public static final String HEADER_CPUS = "CPUs"; //$NON-NLS-1$

    /**
     * Construct a UST statistics tree.
     * Add the CPU category to the base statistics tree from TmfBaseStatisticsData
     */
    public CustomStatisticsModel() {
        super();
        Map<String, Set<String>> keys = getKeys();

        // //////////// Adding category sets
        keys.put(HEADER_CPUS, new HashSet<String>());

        // /////////// Adding value sets
        // Under a trace
        // Add the CPU category to the root node
        Set<String> temp = keys.get(ROOT_NODE_KEY);
        if (temp == null) {
            temp = new HashSet<String>();
        }
        temp.add(HEADER_CPUS);
        keys.put(ROOT_NODE_KEY, temp);
        // Under a CPU
        temp = new HashSet<String>(4);
        temp.add(HEADER_EVENT_TYPES);
        keys.put(mergeString(HEADER_CPUS, NODE), temp);
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.linuxtools.tmf.ui.views.statistics.model.TmfBaseStatisticsData#getNormalPaths(org.eclipse.linuxtools.tmf.event.TmfEvent,
     * org.eclipse.linuxtools.tmf.ui.views.statistics.ITmfEventInfo)
     */
    @Override
    @SuppressWarnings("unchecked")
    protected TmfFixedArray<String>[] getNormalPaths(TmfEvent event, ITmfExtraEventInfo extraInfo) {
        String trace = extraInfo.getTraceName();
        LttngEvent lttngEvent = (LttngEvent) event;
        String cpus = String.valueOf(lttngEvent.getCpuId());
        
        List<TmfFixedArray<String>> paths = new ArrayList<TmfFixedArray<String>>(Arrays.asList(super.getNormalPaths(event, extraInfo)));

        paths.add(new TmfFixedArray<String>(trace, HEADER_CPUS, cpus));
        return paths.toArray(new TmfFixedArray[0]);
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.linuxtools.tmf.ui.views.statistics.model.TmfBaseStatisticsData#getTypePaths(org.eclipse.linuxtools.tmf.event.TmfEvent,
     * org.eclipse.linuxtools.tmf.ui.views.statistics.ITmfEventInfo)
     */
    @Override
    @SuppressWarnings("unchecked")
    protected TmfFixedArray<String>[] getTypePaths(TmfEvent event, ITmfExtraEventInfo extraInfo) {
        String trace = extraInfo.getTraceName();
        LttngEvent lttngEvent = (LttngEvent) event;
        String cpus = String.valueOf(lttngEvent.getCpuId());

        List<TmfFixedArray<String>> paths = new ArrayList<TmfFixedArray<String>>(Arrays.asList(super.getTypePaths(event, extraInfo)));
        
        String type = event.getType().toString();
        
        paths.add(new TmfFixedArray<String>(trace, HEADER_CPUS, cpus, HEADER_EVENT_TYPES, type));
        return paths.toArray(new TmfFixedArray[0]);
    }
}    
